@@ -12,6 +12,12 @@ NOVA_DIR=${NOVA_DIR:-$DEST/nova}
 NOVA_CONF_DIR=${NOVA_CONF_DIR:-/etc/nova}
 NOVA_CONF=${NOVA_CONF:-NOVA_CONF_DIR/nova.conf}
 
+# Configure LXD storage backends
+LXD_BACKEND_DRIVER=${LXD_BACKEND_DRIVER:default}
+LXD_DISK_IMAGE=${DATA_DIR}/lxd.img
+LXD_ZFS_ZPOOL=devstack
+LXD_LOOPBACK_DISK_SIZE=${LXD_LOOPBACK_DISK_SIZE:-8G}
+
 # nova-lxd directories
 NOVA_COMPUTE_LXD_DIR=${NOVA_COMPUTE_LXD_DIR:-${DEST}/nova-lxd}
 NOVA_COMPUTE_LXD_PLUGIN_DIR=$(readlink -f $(dirname ${BASH_SOURCE[0]}))
@@ -20,12 +26,19 @@ NOVA_COMPUTE_LXD_PLUGIN_DIR=$(readlink -f $(dirname ${BASH_SOURCE[0]}))
 GLANCE_CONF_DIR=${GLANCE_CONF_DIR:-/etc/glance}
 GLANCE_API_CONF=$GLANCE_CONF_DIR/glance-api.conf
 
-source $NOVA_COMPUTE_LXD_PLUGIN_DIR/nova-lxd-functions.sh
-
 function pre_install_nova-lxd() {
     # Install OS packages if necessary with "install_package ...".
-    install_lxd
-    install_pylxd
+    echo_summary "Installing LXD"
+    if is_ubuntu; then
+        if [ "$DISTRO" == "trusty" ]; then
+            sudo add-apt-repository -y ppa:ubuntu-lxc/lxd-stable
+        fi
+        if ! ( is_package_installed lxd ); then
+            install_package lxd
+        fi
+
+        add_user_to_group $STACK_USER $LXD_GROUP
+    fi
 }
 
 function install_nova-lxd() {
@@ -38,8 +51,10 @@ function configure_nova-lxd() {
     iniset $NOVA_CONF DEFAULT compute_driver lxd.LXDDriver
     iniset $NOVA_CONF DEFAULT force_config_drive False
 
-    iniset $GLANCE_API_CONF DEFAULT disk_formats "ami,ari,aki,vhd,raw,iso,qcow2,root-tar"
-    iniset $GLANCE_API_CONF DEFAULT container_formats "ami,ari,aki,bare,ovf,tgz"
+    if is_service_enabled glance; then
+        iniset $GLANCE_API_CONF DEFAULT disk_formats "ami,ari,aki,vhd,raw,iso,qcow2,root-tar"
+        iniset $GLANCE_API_CONF DEFAULT container_formats "ami,ari,aki,bare,ovf,tgz"
+    fi
 
     # Install the rootwrap
     sudo install -o root -g root -m 644 $NOVA_COMPUTE_LXD_DIR/etc/nova/rootwrap.d/*.filters $NOVA_CONF_DIR/rootwrap.d
@@ -47,22 +62,33 @@ function configure_nova-lxd() {
 
 function init_nova-lxd() {
     # Initialize and start the service.
-    
-    mkdir -p mkdir -p $TOP_DIR/files
 
-    # Download and install the root-tar image from xenial
-    wget --progress=dot:giga -c https://cloud-images.ubuntu.com/xenial/current/xenial-server-cloudimg-amd64-root.tar.gz  \
-         -O $TOP_DIR/files/xenial-server-cloudimg-amd64-root.tar.gz
-    openstack --os-cloud=devstack-admin --os-region-name="$REGION_NAME" image create "ubuntu-16.04-lxd-root" \
-         --public --container-format bare --disk-format raw < $TOP_DIR/files/xenial-server-cloudimg-amd64-root.tar.gz
+    mkdir -p $TOP_DIR/files
 
     # Download and install the cirros lxc image
-    wget --progress=dot:giga -c http://download.cirros-cloud.net/${CIRROS_VERSION}/cirros-${CIRROS_VERSION}-${CIRROS_ARCH}-lxc.tar.gz \
-        -O $TOP_DIR/files/cirros-${CIRROS_VERSION}-${CIRROS_ARCH}-lxc.tar.gz
-    openstack --os-cloud=devstack-admin --os-region-name="$REGION_NAME" image create "cirros-${CIRROS_VERSION}-${CIRROS_ARCH}-lxd" \
-        --public --container-format bare --disk-format raw < $TOP_DIR/files/cirros-${CIRROS_VERSION}-${CIRROS_ARCH}-lxc.tar.gz
- 
+    CIRROS_IMAGE_FILE=cirros-${CIRROS_VERSION}-${CIRROS_ARCH}-lxc.tar.gz
+    if [ ! -f $TOP_DIR/files/$CIRROS_IMAGE_FILE ]; then
+        wget --progress=dot:giga \
+             -c http://download.cirros-cloud.net/${CIRROS_VERSION}/${CIRROS_IMAGE_FILE} \
+             -O $TOP_DIR/files/${CIRROS_IMAGE_FILE}
+    fi
+    openstack --os-cloud=devstack-admin \
+              --os-region-name="$REGION_NAME" image create "cirros-${CIRROS_VERSION}-${CIRROS_ARCH}-lxd" \
+              --public --container-format bare \
+              --disk-format raw < $TOP_DIR/files/cirros-${CIRROS_VERSION}-${CIRROS_ARCH}-lxc.tar.gz
+
     if is_service_enabled tempest; then
+        # Download and install the root-tar image from xenial
+        UBUNTU_IMAGE_FILE=xenial-server-cloudimg-amd64-root.tar.gz
+        if [ ! -f $TOP_DIR/files/$UBUNTU_IMAGE_FILE ]; then
+            wget --progress=dot:giga -c \
+                 https://cloud-images.ubuntu.com/xenial/current/xenial-server-cloudimg-amd64-root.tar.gz \
+                 -O $TOP_DIR/files/xenial-server-cloudimg-amd64-root.tar.gz
+        fi
+        openstack --os-cloud=devstack-admin --os-region-name="$REGION_NAME" image create "ubuntu-16.04-lxd-root" \
+                  --public --container-format bare \
+                  --disk-format raw < $TOP_DIR/files/${UBUNTU_IMAGE_FILE}
+
        TEMPEST_CONFIG=${TEMPEST_CONFIG:-$TEMPEST_DIR/etc/tempest.conf}
        TEMPEST_IMAGE=`openstack image list | grep cirros-0.3.4-x86_64-lxd | awk {'print $2'}` 
        TEMPEST_IMAGE_ALT=`openstack image list | grep ubuntu-16.04-lxd-root | awk {'print $2'}`
@@ -73,7 +99,6 @@ function init_nova-lxd() {
        iniset $TEMPEST_CONFIG compute-feature-enabled attach_encrypted_volume False
        iniset $TEMPEST_CONFIG compute image_ref $TEMPEST_IMAGE
        iniset $TEMPEST_CONFIG compute image_ref_alt $TEMPEST_IMAGE_ALT
-       iniset $TEMPEST_CONFIG validation run_validation True
     fi
 
     if is_service_enabled cinder; then
@@ -82,6 +107,22 @@ function init_nova-lxd() {
     fi
 }
 
+function configure_lxd_block() {
+   echo_summary "Configure LXD storage backend"
+   if is_ubuntu; then
+      if [ "$LXD_BACKEND_DRIVER" == "default" ]; then
+         echo "Nothing to be done"
+      elif [ "$LXD_BACKEND_DRIVER" == "zfs" ]; then
+         echo "Configuring ZFS backend"
+         truncate -s $LXD_LOOPBACK_DISK_SIZE $LXD_DISK_IMAGE
+         sudo apt-get install -y zfs
+         lxd_dev=`sudo losetup --show -f ${LXD_DISK_IMAGE}`
+         sudo lxd init --auto --storage-backend zfs --storage-pool $LXD_ZFS_ZPOOL \
+              --storage-create-device $lxd_dev
+      fi
+   fi
+}
+
 function shutdown_nova-lxd() {
     # Shut the service down.
     :
@@ -89,7 +130,9 @@ function shutdown_nova-lxd() {
 
 function cleanup_nova-lxd() {
     # Cleanup the service.
-    :
+    if [ "$LXD_BACKEND_DRIVER" == "zfs" ]; then
+       sudo zpool destroy ${LXD_ZFS_ZPOOL}
+    fi
 }
 
 if is_service_enabled nova-lxd; then
@@ -98,6 +141,7 @@ if is_service_enabled nova-lxd; then
         # Set up system services
         echo_summary "Configuring system services nova-lxd"
         pre_install_nova-lxd
+        configure_lxd_block
 
     elif [[ "$1" == "stack" && "$2" == "install" ]]; then
         # Perform installation of service source
